package filter;

import grid.GridAPI;
import grid.GridFactory;

import java.io.IOException;

import javax.servlet.Filter;
import javax.servlet.FilterChain;
import javax.servlet.FilterConfig;
import javax.servlet.ServletException;
import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;
import javax.servlet.http.Cookie;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import log.CLog;
import wrapper.GridHttpServletRequest;
import wrapper.GridHttpSession;

public class HttpFilter implements Filter {

	final String JDGSESSION = "JDGSESSION";
	
	public void destroy() {
		CLog.debug( this, "Destroy !! ");
	}

	public void doFilter(ServletRequest req, ServletResponse resp,
			FilterChain chain) throws IOException, ServletException {
		CLog.debug( this, "doFilter() !! " );
		
        HttpServletRequest request = (HttpServletRequest) req;
        HttpServletResponse response = (HttpServletResponse) resp;
		
        if ( !( request instanceof GridHttpServletRequest ) ) {
        	request = new GridHttpServletRequest( request );
        }

        String jdgsession = null;
        
        // Check if we have a session in the cookie
        Cookie[] cookies = request.getCookies();
        if ( cookies != null ) {        	        	
        	for ( Cookie c : cookies ) {
        		if ( c.getName().equalsIgnoreCase( JDGSESSION ) ) {
        			jdgsession = c.getValue();
        			CLog.debug( this, "Old JDGSESSION: " + jdgsession );
        			((GridHttpServletRequest)request).setJdgsession( jdgsession );   
        			((GridHttpSession) request.getSession()).checkAndPopulateSession();        			
        		}
        	}
        }
    	// New JDGSession
    	if (jdgsession == null) {
    		GridAPI grid = GridFactory.getInstance();
    		jdgsession = grid.getNewUUID().toString();
    		Cookie jdgcookie = new Cookie( "JDGSESSION", jdgsession );
    		response.addCookie( jdgcookie );
    		CLog.debug( this, "New JDGSESSION: " + jdgsession );
    		((GridHttpServletRequest)request).setJdgsession( jdgsession );
    	}
        
        // Change the original request by the wrapper
		chain.doFilter( request, response );
	}

	public void init(FilterConfig cfg) throws ServletException {
		CLog.debug( this, "Init !! " );
		String gridUrl = cfg.getInitParameter( "jdg-url" );
		if ( gridUrl != null )
			System.setProperty( "infinispan.client.hotrod.server_list", gridUrl );
		else
			System.setProperty( "infinispan.client.hotrod.server_list", "localhost:11222" );
	}

}
